package Conquest;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Hashtable;

/**
 * Si occupa del processo di manipolazione dei dati per la fase di 
 * pre-processing per la creazione della mining view
 *
 */
public class SpqlManager {
	
	/**
	 * Determina se il filtro di partizione delle query debba essere usato
	 * oppure no
	 */
	public boolean FILTRO = false;
	
	private boolean verbose = false;
	
	private ArrayList Attrtype = new ArrayList(0);
	
	private String DB_URL = null;
	
	private String DB_DRIVER = null;
	
	private String DB_PASSWORD = null;
	
	private String DB_UTENTE = null;
	
	public ArrayList resultDataset = new ArrayList(0);
	
	public ArrayList resultAttribute = null;
	
	public Hashtable resultMapAttribute = null;
	
	public ArrayList resultMapItem = new ArrayList(0);
	
	public ArrayList resolveAttribute = new ArrayList(0);
	
	private ArrayList Select = null;
	
	public int Mfactor = 1;
	
	int index = 0;
	
	public int Pvalue = 0;
	
	public int Pmax = 0;
	
	double Tstart = 0;
	
	private String nomefile = null;
	
	private String attname = null;
	
	static final public String allDistinct = "(All Distinct Transaction)";
	
	private boolean OnlyAttribute = false;
	
	/**
	 * Viene istanziato un nuovo oggetto che gestisce le comunicazioni e
	 * l'esecuzione di query al database definito dall'url
	 */
	
	public SpqlManager() {
	}
	
	/**
	 *  Vengono settate le informazioni di connessione
	 * @param Url l'indirizzo del DB
	 * @param DBDriver i driver del DB
	 * @param Utente il nume utente per l'accesso
	 * @param Password la password per l'accesso
	 */
	public SpqlManager(String Url, String DBDriver, String Utente,
			String Password) {
		DB_URL = Url;
		DB_DRIVER = DBDriver;
		DB_UTENTE = Utente;
		DB_PASSWORD = Password;
	}
	
	/**
	 * Resetta le preferenze dell'oggetto
	 */
	public void clear() {
		index = 0;
		FILTRO = false;
		OnlyAttribute = false;
		resolveAttribute.clear();
	}
	
	/**
	 * setta i metodi di risoluzione conflitti degli attributi
	 * 
	 * @param set l'elenco dei metodi
	 */
	public void resolveAttributeMethod(ArrayList set) {
		resolveAttribute = set;
	}
	
	/**
	 * Attiva il glag per la sola generazione di attributi
	 */
	public void doOnlyAttribute() {
		OnlyAttribute = true;
	}
	
	/**
	 * Attiva il partizionamento di una query SPQL
	 */
	public void enableSuddivision() {
		FILTRO = true;
	}
	
	/**
	 * Esegue una query e genera i due file in formato fimi per il dataset e il
	 * gli attributi. L'array di Stringhe deve contenere: 0 - nome file query 1 -
	 * file di destinazione
	 */
	public boolean execute(String[] args, boolean fromfile, boolean ver,
			int limit) {
		nomefile = args[1];
		attname = args[2];
		verbose = ver;
		Tstart = System.currentTimeMillis();
		if (verbose)
			System.out.println("Tempo: "
					+ (System.currentTimeMillis() - Tstart) / 1000);
		Attrtype.clear();
		resultDataset.clear();
		resultMapItem.clear();
		if (args.length != 3) {
			if (verbose)
				System.out
				.println("Errore: Parametri non sufficenti per eseguire la query");
			return false;
		}
		Connection connection = null;
		Statement statement = null;
		
		String s = new String();
		if (fromfile)
			s = leggiQuery(args[0]);
		else
			s = args[0];
		if (s == "ERRORE") {
			if (verbose)
				System.out.println("Errore: File Query Inesistente");
			return false;
		}
		
		String[] sSqlE = new String[7];
		sSqlE = takeSqlE(s);
		ArrayList ExtTransaction = new ArrayList(0);
		ArrayList ExtItem = new ArrayList(0);
		ArrayList ExtAttribute = new ArrayList(0);
		Select = new ArrayList(0);
		ArrayList OriginalSelect = new ArrayList(0);
		if (verbose)
			System.out.println("Tempo: "
					+ (System.currentTimeMillis() - Tstart) / 1000);
		
		ExtTransaction = extCampi(sSqlE[1]);
		ExtItem = extCampi(sSqlE[2]);
		ExtAttribute = extCampi(sSqlE[3]);
		OriginalSelect = setOriginal(extCampi(sSqlE[4]));
		Select = setRename(extCampi(sSqlE[4]));
		
		try {
			Class.forName(DB_DRIVER);
			connection = DriverManager.getConnection(DB_URL, DB_UTENTE,
					DB_PASSWORD);
			statement = connection.createStatement();
			
		} catch (Exception e) {
			if (verbose)
				System.out.println("Errore durante la connessione al database"
						+ e);
			return false;
		}
		
		ArrayList Query = new ArrayList(0);
		if (ExtAttribute.size() != 0) {
			for (int o = 0; o < ExtAttribute.size(); o++)
				for (int i = 0; i < ExtItem.size(); i++) {
					String query = null;
					int c = 0;
					try {
						c = ((Integer) resolveAttribute.get(o)).intValue();
					} catch (Exception ex) {
						c = 0;
					}
					switch (c) {
					case 3:
						query = "SELECT "
							+ ExtItem.get(i)
							+ ", MAX("
							+ ExtAttribute.get(o)
							+ ")\n"
							+ sSqlE[0].substring(sSqlE[0].indexOf("FROM"),
									sSqlE[0].indexOf("ORDER BY"))
									+ "\nGROUP BY " + ExtItem.get(i);
						break;
					case 4:
						query = "SELECT "
							+ ExtItem.get(i)
							+ ", MIN("
							+ ExtAttribute.get(o)
							+ ")\n"
							+ sSqlE[0].substring(sSqlE[0].indexOf("FROM"),
									sSqlE[0].indexOf("ORDER BY"))
									+ "\nGROUP BY " + ExtItem.get(i);
						break;
					case 5:
						query = "SELECT "
							+ ExtItem.get(i)
							+ ", AVG("
							+ ExtAttribute.get(o)
							+ ")\n"
							+ sSqlE[0].substring(sSqlE[0].indexOf("FROM"),
									sSqlE[0].indexOf("ORDER BY"))
									+ "\nGROUP BY " + ExtItem.get(i);
						break;
					case 6:
						query = "SELECT "
							+ ExtItem.get(i)
							+ ", SUM("
							+ ExtAttribute.get(o)
							+ ")\n"
							+ sSqlE[0].substring(sSqlE[0].indexOf("FROM"),
									sSqlE[0].indexOf("ORDER BY"))
									+ "\nGROUP BY " + ExtItem.get(i);
						break;
					case 2:
						query = "SELECT "
							+ ExtItem.get(i)
							+ ", COUNT("
							+ ExtAttribute.get(o)
							+ ")\n"
							+ sSqlE[0].substring(sSqlE[0].indexOf("FROM"),
									sSqlE[0].indexOf("ORDER BY"))
									+ "\nGROUP BY " + ExtItem.get(i);
						break;
					default:
						try {
							query = "SELECT "
								+ ExtItem.get(i)
								+ ", "
								+ ExtAttribute.get(o)
								+ "\n"
								+ sSqlE[0].substring(sSqlE[0]
								                           .indexOf("FROM"), sSqlE[0]
								                                                   .indexOf("ORDER BY"));
							break;
						} catch (Exception ex) {
							query = "SELECT "
								+ ExtItem.get(i)
								+ ", "
								+ ExtAttribute.get(o)
								+ "\n"
								+ sSqlE[0]
								        .substring(
								        		sSqlE[0].indexOf("FROM"),
								        		sSqlE[0].length());
							break;
						}
					}
					Query.add(query);
				}
			resultMapItem = new ArrayList(0);
			resultMapAttribute = new Hashtable(0);
			resultAttribute = new ArrayList(0);
			
			if (!validation(ExtTransaction, Select)) {
				if (verbose)
					System.out.println("EXTEND non valida (TRANSACTION)");
				return false;
			}
			if (!validation(ExtItem, Select)) {
				if (verbose)
					System.out.println("EXTEND non valida (ITEM)");
				return false;
			}
			if (!validation(ExtAttribute, Select)) {
				if (verbose)
					System.out.println("EXTEND non valida (ATTRIBUTE)");
				return false;
			}
			
			genAttribute(Query, statement, ExtItem, ExtAttribute);
			ExtAttribute.clear();
			if (OnlyAttribute)
				return true;
		} else {
			for (int i = 0; i < ExtItem.size(); i++) {
				try {
					Query.add("SELECT "
							+ ExtItem.get(i)
							+ "\n"
							+ sSqlE[0].substring(sSqlE[0].indexOf("FROM"),
									sSqlE[0].indexOf("ORDER BY")));
				} catch (Exception ex) {
					Query.add("SELECT "
							+ ExtItem.get(i)
							+ "\n"
							+ sSqlE[0].substring(sSqlE[0].indexOf("FROM"),
									sSqlE[0].length()));
				}
			}
			if (!validation(ExtTransaction, Select)) {
				if (verbose)
					System.out.println("EXTEND non valida (TRANSACTION)");
				return false;
			}
			if (!validation(ExtItem, Select)) {
				if (verbose)
					System.out.println("EXTEND non valida (ITEM)");
				return false;
			}
			if (!validation(ExtAttribute, Select)) {
				if (verbose)
					System.out.println("EXTEND non valida (ATTRIBUTE)");
				return false;
			}
			resultMapItem = new ArrayList(0);
			resultMapAttribute = new Hashtable(0);
			resultAttribute = new ArrayList(0);
			
			genOnlyItem(Query, statement, ExtItem);
			if (OnlyAttribute)
				return true;
		}
		
		if (verbose)
			System.out.println("Tempo: "
					+ (System.currentTimeMillis() - Tstart) / 1000);
		
		if (verbose)
			System.out.println("Using: \n" + s);
		
		Query = new ArrayList(0);
		try {
			if (FILTRO)
				divideQuery(sSqlE[0],
						(String) OriginalSelect.get(((Integer) ExtTransaction
								.get(index)).intValue() - 1), Query, statement);
			else
				Query.add(sSqlE[0]);
			if (verbose)
				System.out.println("Tempo: "
						+ (System.currentTimeMillis() - Tstart) / 1000);
			statement = connection.createStatement();
			System.gc();
			Pvalue = 0;
			Pmax = Query.size() * 3;
			for (int i = 0; i < Query.size(); i++) {
				Pvalue++;
				if (verbose)
					System.out.println("\nProcessing: (" + (i + 1) + "/"
							+ Query.size() + ")\n" + (String) Query.get(i));
				ResultSet resultset = null;
				try {
					if (limit > 0)
						resultset = statement.executeQuery(((String) Query
								.get(i)).trim()
								+ "\nLIMIT " + limit);
					else
						resultset = statement.executeQuery(((String) Query
								.get(i)).trim());
				} catch (Exception ex) {
					resultset = statement.executeQuery(((String) Query.get(i))
							.trim());
				}
				if (verbose)
					System.out.println("Tempo: "
							+ (System.currentTimeMillis() - Tstart) / 1000);
				if (verbose)
					System.out
					.println("Generazione Dataset e Attributi in corso... ");
				Pvalue++;
				genDataset(ExtTransaction, ExtItem, resultset);
				Pvalue++;
				if (verbose)
					System.out.println("Tempo: "
							+ (System.currentTimeMillis() - Tstart) / 1000);
			}
			
			if (connection != null)
				connection.close();
		} catch (OutOfMemoryError e) {
			System.out.println("outOfMemory" + e);
			if (FILTRO == false) {
				if (verbose)
					System.out
					.println("\n\nSuddivisione della query attivata\n\n");
				FILTRO = true;
				execute(args, fromfile, verbose, limit);
				return false;
			} else if (index < ExtTransaction.size()) {
				if (verbose)
					System.out
					.println("\n\nRiavvio con modifica suddivisione\n\n");
				index++;
				execute(args, fromfile, verbose, limit);
				return false;
			} else {
				if (verbose)
					System.out
					.println("Memoria insufficiente per eseguire l'operazione richiesta");
				return false;
			}
		} catch (Exception e) {
			e.printStackTrace();
			if (verbose)
				System.out
				.println("Errore eseguendo la Query o chiudendo la connessione"
						+ e);
			return false;
		}
		return true;
	}
	
	/**
	 * Legge la Query dal file
	 */
	private String leggiQuery(String nomefile) {
		BufferedReader br = null;
		FileReader r = null;
		try {
			r = new FileReader(nomefile);
			br = new BufferedReader(r);
		} catch (Exception e) {
			if (verbose)
				System.out.println("Errore aprendo il file della Query" + e);
			return "ERRORE";
		}
		String s = "";
		String c = "";
		try {
			do {
				c = br.readLine();
				if (c != null)
					s = s + c + " ";
			} while (c != null);
			r.close();
		} catch (Exception e) {
			if (verbose)
				System.out.println("Errore leggendo il file della Query" + e);
			return "ERRORE";
		}
		s = s.trim();
		return s;
	}
	
	/**
	 * Crea sottostringhe partendo dalla query per il parsing: 0 - La sola query
	 * sql interna alla query spql 1 - La sequenza di campi per le transazioni 2 -
	 * La sequenza di campi per gli item 3 - La sequenza di campi per gli
	 * attributi 4 - La sola sequenza dei campi select della query sql interna
	 * alla query spql 5 - la soglia sigma 6 - La sequenza di funzioni per il
	 * constrained
	 */
	public String[] takeSqlE(String s) {
		String[] ris = new String[7];
		String[] temp = new String[2];
		try {
			temp = s.split("TRANSACTION", 2);
			ris[0] = temp[0].substring(temp[0].indexOf("\nSELECT"),
					temp[0].length()).trim();
			ris[1] = temp[1].substring(0, temp[1].indexOf("\nITEM")).replace(
					',', ' ').trim();
			ris[2] = temp[1].substring(temp[1].indexOf("\nITEM") + 5,
					temp[1].indexOf("ATTRIBUTE")).replace(',', ' ').trim();
			ris[3] = temp[1].substring(temp[1].indexOf("\nATTRIBUTE") + 10,
					temp[1].indexOf("CONSTRAINED")).replace(',', ' ').trim();
			ris[4] = temp[0].substring(temp[0].indexOf("\nSELECT") + 7,
					temp[0].indexOf("FROM")).replace(',', ' ').trim();
			ris[5] = temp[0].substring(temp[0].indexOf("SUPP>=") + 6,
					temp[0].indexOf("IN", 5)).trim();
			ris[6] = temp[1].substring(
					temp[1].indexOf("\nCONSTRAINED BY") + 15, temp[1].length())
					.replace(',', ' ').trim();
		} catch (Exception ex) {
			ris = null;
		}
		return ris;
	}
	
	/**
	 * Da una riga rappresentante una sequenza estre la sequenza e la inserisce
	 * in un Array
	 */
	public ArrayList extCampi(String transaction) {
		ArrayList listCampi = new ArrayList(0);
		transaction = transaction.trim();
		String[] temp = new String[2];
		transaction = transaction.trim();
		if ((transaction.compareTo(" ") == 0)
				|| (transaction.compareTo("") == 0))
			return listCampi;
		while (transaction.indexOf(" ") != -1) {
			temp = transaction.split(" ", 2);
			transaction = temp[1].trim();
			listCampi.add(temp[0]);
		}
		listCampi.add(transaction.trim());
		return listCampi;
	}
	
	/**
	 * Controlla se i campi di unit sono presenti in list (usata per la
	 * validazione delle transazioni, item e attributi, che devono essere
	 * contenuti nella select). Inoltre trasforma i nomi di capi in posizioni
	 * nella lista (+1 perche poi il conteggio  in base 1 --- n e non 0 --- n)
	 */
	public boolean validation(ArrayList unit, ArrayList list) {
		for (int i = 0; i < unit.size(); i++) {
			if (unit.get(i).toString().compareTo(allDistinct) == 0) {
				unit.add(i, new Integer(-1));
				unit.remove(i + 1);
			} else if (unit.get(i).toString().compareTo("") != 0) {
				if (!list.contains(unit.get(i)))
					return false;
				else {
					int pos = list.indexOf(unit.get(i));
					unit.add(i, new Integer(pos + 1));
					unit.remove(i + 1);
				}
			}
		}
		return true;
	}
	
	/**
	 * A partire dal risultato di una Query viene generato il dataset
	 */
	private void genDataset(ArrayList TUnit, ArrayList IUnit, ResultSet result) {
		FileWriter Transactionf = null;
		BufferedWriter bTransactionf = null;
		
		try {
			Transactionf = new FileWriter(nomefile + ".txt");
			bTransactionf = new BufferedWriter(Transactionf);
		} catch (Exception e) {
			e.printStackTrace();
		}
		;
		
		try {
			int numberOfColumns = result.getMetaData().getColumnCount();
			ArrayList Transaction = new ArrayList(0);
			String currentKey = null;
			while (result.next()) {
				String key = "";
				ArrayList value = new ArrayList(0);
				String t = "";
				for (int i = 1; i <= numberOfColumns; i++) {
					t = result.getString(i);
					if (TUnit.contains(new Integer(i))) {
						if (key != "")
							key = key + "_";
						key = key + t;
					}
					if (IUnit.contains(new Integer(i))) {
						String v = t + "(_" + i + "_)";
						value.add(v);
					}
				}
				if (currentKey == null)
					currentKey = key;
				if (currentKey.compareTo(key) != 0) {
					String cur = "";
					for (int item = 0; item < Transaction.size(); item++)
						if (item != Transaction.size() - 1) {
							bTransactionf.write((Transaction.get(item)
									.toString() + " "));
							cur += Transaction.get(item).toString() + " ";
						} else {
							bTransactionf.write((Transaction.get(item)
									.toString()));
							cur += Transaction.get(item).toString();
						}
					resultDataset.add(cur);
					bTransactionf.newLine();
					Transaction.clear();
					currentKey = key;
				}
				for (int i = 0; i < value.size(); i++)
					if (!Transaction.contains(value.get(i)))
						Transaction.add(new Integer(resultMapItem
								.indexOf(Select
										.get(Integer.valueOf(
												IUnit.get(i).toString())
												.intValue() - 1)
												+ ":" + value.get(i))));
			}
			// ultima trasazione//
			String cur = "";
			for (int item = 0; item < Transaction.size(); item++)
				if (item != Transaction.size() - 1) {
					bTransactionf
					.write((Transaction.get(item).toString() + " "));
					cur += Transaction.get(item).toString() + " ";
				} else {
					bTransactionf.write((Transaction.get(item).toString()));
					cur += Transaction.get(item).toString();
				}
			resultDataset.add(cur);
			bTransactionf.newLine();
			Transaction.clear();
			bTransactionf.close();
			Transactionf.close();
		} catch (Exception e) {
			e.printStackTrace();
			return;
		}
	}
	
	/**
	 * Costruisce una partizione della query iniziale
	 */
	private void divideQuery(String sqlbase, String Key, ArrayList Query,
			Statement statement) {
		try {
			String getKeyQuery = new String();
			getKeyQuery = "SELECT DISTINCT "
				+ Key
				+ " "
				+ sqlbase.substring(sqlbase.indexOf("FROM"), sqlbase
						.length()) + ";";
			if (verbose)
				System.out.println("Processing: \n" + getKeyQuery);
			ResultSet result = statement.executeQuery(getKeyQuery);
			String T = "'";
			if ((result.getMetaData().getColumnType(1) == 4)
					|| (result.getMetaData().getColumnType(1) == 5)
					|| (result.getMetaData().getColumnType(1) == -6))
				T = "";
			if ((result.getMetaData().getColumnType(1) == -5)
					|| (result.getMetaData().getColumnType(1) == 3)
					|| (result.getMetaData().getColumnType(1) == 8)
					|| (result.getMetaData().getColumnType(1) == 6)
					|| (result.getMetaData().getColumnType(1) == 2))
				T = "";
			if (sqlbase.indexOf("WHERE") == -1)
				while (result.next())
					Query.add(sqlbase + " WHERE " + Key + "=" + T
							+ result.getString(1) + T + ";");
			else {
				String a = sqlbase.substring(sqlbase.indexOf("SELECT"), sqlbase
						.indexOf("WHERE") + 5);
				String b = sqlbase.substring(sqlbase.indexOf("WHERE") + 5,
						sqlbase.length());
				while (result.next())
					Query.add(a + " " + Key + "=" + T + result.getString(1) + T
							+ " AND" + b + ";");
			}
		} catch (Exception e) {
			if (verbose)
				System.out.println("Errore generando la lista di Query " + e);
			return;
		}
	}
	
	/**
	 * Sostituisce nell'elenco della select tutti i rename del tipo .... AS ....
	 * con lo pseudonimo
	 */
	public ArrayList setRename(ArrayList select) {
		while (select.indexOf("AS") >= 0) {
			int i = select.indexOf("AS");
			select.remove(i);
			select.remove(i - 1);
		}
		return select;
	}
	
	/**
	 * Sostituisce nell'elenco della select tutti i rename del tipo .... AS ....
	 * con il nome originale del campo
	 */
	public ArrayList setOriginal(ArrayList select) {
		while (select.indexOf("AS") >= 0) {
			int i = select.indexOf("AS");
			select.remove(i + 1);
			select.remove(i);
		}
		return select;
	}
	
	/**
	 * Genera gli attributi e gli Item e i loro rispettivi mapping
	 * 
	 * @param Query l'inieme di query da eseguire
	 * @param statement la connessione al DB
	 * @param ExtItem Le posiozioni degli item nella query SQL interna
	 * @param ExtAttribute Le posizioni degli attributi nella quesy SQL interna
	 */
	public void genAttribute(ArrayList Query, Statement statement,
			ArrayList ExtItem, ArrayList ExtAttribute) {
		FileWriter MapItemf = null;
		BufferedWriter bMapItemf = null;
		FileWriter MapAttributef = null;
		BufferedWriter bMapAttributef = null;
		FileWriter Attributef = null;
		BufferedWriter bAttributef = null;
		try {
			MapItemf = new FileWriter(nomefile + ".imap");
			bMapItemf = new BufferedWriter(MapItemf);
			MapAttributef = new FileWriter(nomefile + "_" + attname + ".amap");
			bMapAttributef = new BufferedWriter(MapAttributef);
			Attributef = new FileWriter(nomefile + "_" + attname + ".att");
			bAttributef = new BufferedWriter(Attributef);
		} catch (Exception e) {
			e.printStackTrace();
		}
		;
		int idi = 0;
		int ida = 0;
		Pmax = Query.size();
		try {
			int i = 0;
			int p = 0;
			boolean numeric = true;
			Hashtable reverse = new Hashtable(0);
			for (int o = 0; o < ExtAttribute.size(); o++) {
				ArrayList list = new ArrayList(0);
				resultMapItem.clear();
				for (int a = 0; a < ExtItem.size(); a++) {
					ResultSet result = null;
					Pvalue++;
					if (verbose)
						System.out.println("Eseguo query " + i + " Tempo: "
								+ (System.currentTimeMillis() - Tstart) / 1000);
					result = statement.executeQuery(((String) Query.get(i))
							.trim());
					while (result.next()) {
						String res = result.getString(1);
						String item = (Select.get(Integer.valueOf(
								ExtItem.get(a).toString()).intValue() - 1)
								+ ":" + res + "(_" + ExtItem.get(a) + "_)")
								.trim();
						if (!resultMapItem.contains(item)) {
							resultMapItem.add(item);
							bMapItemf.write(idi + " " + item);
							bMapItemf.newLine();
							idi++;
							String value = null;
							try {
								value = result.getString(2).trim();
							} catch (Exception ex) {
								value = "nil";
							}
							if (!resultMapAttribute.contains(value)) {
								p = ida;
								if (numeric)
									try {
										Double.valueOf(value);
									} catch (Exception ex) {
										numeric = false;
									}
									resultMapAttribute.put((p + ""), value);
									reverse.put(value, new Integer(p));
									bMapAttributef.write(p + " " + value);
									bMapAttributef.newLine();
									ida++;
							} else
								p = ((Integer) reverse.get(value)).intValue();
							list.add(p + "");
						} else // TAKE LAST
							if (Integer.valueOf(resolveAttribute.get(o).toString())
									.intValue() == 1) {
								String value = result.getString(2);
								list.add(resultMapItem.indexOf(item), value);
								list.remove(resultMapItem.indexOf(item) + 1);
							}
						
					}
					i++;
				}
				if (numeric) {
					ArrayList newlist = new ArrayList(0);
					for (int att = 0; att < list.size(); att++)
						newlist.add(Double.valueOf(resultMapAttribute.get(
								list.get(att).toString()).toString()));
					resultMapAttribute.clear();
					list = newlist;
				}
				resultAttribute.add(list);
				for (int att = 0; att < list.size(); att++)
					bAttributef.write(list.get(att).toString() + " ");
				bAttributef.newLine();
			}
			bAttributef.close();
			bMapAttributef.close();
			bMapItemf.close();
			Attributef.close();
			MapAttributef.close();
			MapItemf.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/**
	 * Genera solo gli item e il loro mapping
	 * 
	 * @param Query la lista di query da eseguire
	 * @param statement la connessione al DB
	 * @param ExtItem Le posiozioni degli item nella query SQL interna 
	 */
	public void genOnlyItem(ArrayList Query, Statement statement,
			ArrayList ExtItem) {
		FileWriter MapItemf = null;
		BufferedWriter bMapItemf = null;
		FileWriter MapAttributef = null;
		BufferedWriter bMapAttributef = null;
		FileWriter Attributef = null;
		BufferedWriter bAttributef = null;
		try {
			MapItemf = new FileWriter(nomefile + ".imap");
			bMapItemf = new BufferedWriter(MapItemf);
			MapAttributef = new FileWriter(nomefile + "_" + attname + ".amap");
			bMapAttributef = new BufferedWriter(MapAttributef);
			Attributef = new FileWriter(nomefile + "_" + attname + ".att");
			bAttributef = new BufferedWriter(Attributef);
		} catch (Exception e) {
			e.printStackTrace();
		}
		;
		int idi = 0;
		Pmax = Query.size();
		try {
			resultMapItem.clear();
			for (int a = 0; a < ExtItem.size(); a++) {
				ResultSet result = null;
				Pvalue++;
				if (verbose)
					System.out.println("Eseguo query " + a + " Tempo: "
							+ (System.currentTimeMillis() - Tstart) / 1000);
				result = statement.executeQuery(((String) Query.get(a)).trim());
				while (result.next()) {
					String res = result.getString(1);
					String item = (Select.get(Integer.valueOf(
							ExtItem.get(a).toString()).intValue() - 1)
							+ ":" + res + "(_" + ExtItem.get(a) + "_)").trim();
					if (!resultMapItem.contains(item)) {
						resultMapItem.add(item);
						bMapItemf.write(idi + " " + item);
						bMapItemf.newLine();
						idi++;
					}
				}
			}
			bAttributef.close();
			bMapAttributef.close();
			bMapItemf.close();
			Attributef.close();
			MapAttributef.close();
			MapItemf.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
}
